@@ -17,7 +17,6 @@ package bufconfig
 import (
 	"context"
 	"encoding/json"
-	"fmt"
 	"io"
 
 	"buf.build/go/standard/xslices"
@@ -179,7 +178,7 @@ func readBufGenYAMLFile(
 	case FileVersionV1Beta1:
 		var externalGenYAMLFile externalBufGenYAMLFileV1Beta1
 		if err := getUnmarshalStrict(allowJSON)(data, &externalGenYAMLFile); err != nil {
-			return nil, fmt.Errorf("invalid as version %v: %w", fileVersion, err)
+			return nil, err
 		}
 		generateConfig, err := newGenerateConfigFromExternalFileV1Beta1(externalGenYAMLFile)
 		if err != nil {
@@ -194,7 +193,7 @@ func readBufGenYAMLFile(
 	case FileVersionV1:
 		var externalGenYAMLFile externalBufGenYAMLFileV1
 		if err := getUnmarshalStrict(allowJSON)(data, &externalGenYAMLFile); err != nil {
-			return nil, fmt.Errorf("invalid as version %v: %w", fileVersion, err)
+			return nil, err
 		}
 		generateConfig, err := newGenerateConfigFromExternalFileV1(externalGenYAMLFile)
 		if err != nil {
@@ -209,7 +208,7 @@ func readBufGenYAMLFile(
 	case FileVersionV2:
 		var externalGenYAMLFile externalBufGenYAMLFileV2
 		if err := getUnmarshalStrict(allowJSON)(data, &externalGenYAMLFile); err != nil {
-			return nil, fmt.Errorf("invalid as version %v: %w", fileVersion, err)
+			return nil, err
 		}
 		generateConfig, err := newGenerateConfigFromExternalFileV2(externalGenYAMLFile)
 		if err != nil {
@@ -334,6 +333,7 @@ type externalGenerateManagedConfigV1 struct {
 	GoPackagePrefix     externalGoPackagePrefixConfigV1   `json:"go_package_prefix,omitempty" yaml:"go_package_prefix,omitempty"`
 	ObjcClassPrefix     externalObjcClassPrefixConfigV1   `json:"objc_class_prefix,omitempty" yaml:"objc_class_prefix,omitempty"`
 	RubyPackage         externalRubyPackageConfigV1       `json:"ruby_package,omitempty" yaml:"ruby_package,omitempty"`
+	SwiftPrefix         externalSwiftPrefixConfigV1       `json:"swift_prefix,omitempty" yaml:"swift_prefix,omitempty"`
 	// Override maps from a file option to a file path then to the value.
 	Override map[string]map[string]string `json:"override,omitempty" yaml:"override,omitempty"`
 }
@@ -461,6 +461,20 @@ func (e externalRubyPackageConfigV1) isEmpty() bool {
 	return len(e.Except) == 0 && len(e.Override) == 0
 }
 
+// externalSwiftPrefixConfigV1 represents the swift_prefix config in a v1 buf.gen.yaml file.
+type externalSwiftPrefixConfigV1 struct {
+	Default  string            `json:"default,omitempty" yaml:"default,omitempty"`
+	Except   []string          `json:"except,omitempty" yaml:"except,omitempty"`
+	Override map[string]string `json:"override,omitempty" yaml:"override,omitempty"`
+}
+
+// isEmpty returns true is the config is empty.
+func (e externalSwiftPrefixConfigV1) isEmpty() bool {
+	return e.Default == "" &&
+		len(e.Except) == 0 &&
+		len(e.Override) == 0
+}
+
 // externalObjcClassPrefixConfigV1 represents the objc_class_prefix config in a v1 buf.gen.yaml file.
 type externalObjcClassPrefixConfigV1 struct {
 	Default  string            `json:"default,omitempty" yaml:"default,omitempty"`
